// Standard includes

// Vendor includes

// Project includes
#include "DallasCrc8.hpp"

// Using declarations


// CRC8 tables
const unsigned char DallasCrc8::s_CrcTableLow[] =
{
    0x00, 0x5E, 0xBC, 0xE2, 0x61, 0x3F, 0xDD, 0x83, 0xC2, 0x9C, 0x7E, 0x20, 0xA3, 0xFD, 0x1F, 0x41
};
const unsigned char DallasCrc8::s_CrcTableHigh[] =
{
    0x00, 0x9D, 0x23, 0xBE, 0x46, 0xDB, 0x65, 0xF8, 0x8C, 0x11, 0xAF, 0x32, 0xCA, 0x57, 0xE9, 0x74
};


DallasCrc8::DallasCrc8()
    : m_Crc( 0 ),
      m_CrcInitialValue( 0 )
{
}

DallasCrc8::DallasCrc8( unsigned char crcInitialValue )
    : m_Crc( crcInitialValue ),
      m_CrcInitialValue( crcInitialValue )
{
}

DallasCrc8::~DallasCrc8()
{
}

void DallasCrc8::Restart()
{
    m_Crc = m_CrcInitialValue;
}

unsigned char DallasCrc8::AddByte( unsigned char byte )
{
    unsigned char index = byte ^ m_Crc;
    m_Crc = s_CrcTableLow[ index & 0x0F ] ^ s_CrcTableHigh[ index >> 4 ];

    return m_Crc;
}

unsigned char DallasCrc8::AddBuffer( unsigned char* pBuffer, unsigned int length )
{
    if ( 0 != pBuffer )
    {
        for ( unsigned int i = 0; i < length; i++ )
        {
            this->AddByte( *( pBuffer + i ) );
        }
    }

    return m_Crc;
}

unsigned char DallasCrc8::Get() const
{
    return m_Crc;
}
