#ifndef READERBASE__HPP
#define READERBASE__HPP

// Standard includes

// Vendor includes

// Project includes
#include "Logger.hpp"
#include "UsbSerial.hpp"


class ReaderBase
{
public:
    struct CommandHeader
    {
        unsigned char Code;
        unsigned char Length;
    } __attribute__ ( ( __packed__ ) );

    struct ResponseHeader
    {
        enum
        {
            RESULT__RES_OK = 0x00
        };

        unsigned char Result;
        unsigned char Length;
    } __attribute__ ( ( __packed__ ) );

    ReaderBase( LoggerClient& rLoggerClient, unsigned char* pCommandPacket );
    virtual ~ReaderBase();

    const char* GetNode() const;
    unsigned long GetSerialNumber() const;
    void LogIdentity( LoggerClient& rLoggerClient );

protected:
    struct PacketHeader
    {
        unsigned char StartOfPacket;
        unsigned char TargetStationId;
        unsigned char SourceStationId;
        unsigned char ProtocolOptionCode;
    } __attribute__ ( ( __packed__ ) );

    struct PacketTail
    {
        unsigned char Crc8;
        unsigned char EndOfPacket;
    } __attribute__ ( ( __packed__ ) );

    bool Open( const char* pNode, unsigned long serialNumber, unsigned int speed, char parityCode, unsigned char stopBits );
    bool Reopen();
    void Close();
    void SetReadTimeout( int readTimeoutInMilliseconds );
    bool ProcessCommand( unsigned char* pCommandBuffer, unsigned char* pResponseBuffer, unsigned int responseBufferSize );

private:
    enum
    {
        DEFAULT_READ_TIMEOUT_MILLISECONDS = 1000
    };

    enum PacketData
    {
        PACKET__START = 0x02,
        PACKET__TARGET_STATION_ID = 0x00,
        PACKET__SOURCE_STATION_ID = 0x00,
        PACKET__PROTOCOL_OPTION_CODE = 0x00,
        PACKET__END = 0x03
    };

    // Copy constructor is invalid and not defined, declared here to
    // prevent the compiler generating the default definition
    ReaderBase( const ReaderBase& rOther );

    // Copy assignment operator is invalid and not defined, declared
    // here to prevent compiler generating the default definition
    ReaderBase& operator=( const ReaderBase& rOther );

    LoggerClient& m_rLoggerClient;
    char* m_pNode;
    unsigned long m_SerialNumber;
    struct
    {
        unsigned int Speed;
        char ParityCode;
        unsigned char StopBits;
    } m_PortConfiguration;
    UsbSerial m_UsbSerial;
    unsigned char* m_pCommandPacket;
    PacketTail m_PacketTail;
    int m_ReadTimeoutInMilliseconds;
};


#endif // READERBASE__HPP
