#ifndef READERFABRIC__HPP
#define READERFABRIC__HPP

// Standard includes
#include <pthread.h>

// Vendor includes

// Project includes
#include "ReaderPlugAndPlay.hpp"
#include "ReaderModelPrEh08.hpp"
#include "ReaderModelPrp08.hpp"


class ReaderFabric : protected ReaderPlugAndPlay
{
public:
    ReaderFabric();
    virtual ~ReaderFabric();

    void SetLogger( LoggerServer* pLoggerServer, LoggerClient::LoggerLevel loggerLevel, LoggerClient::LoggerLevel plugAndPlayLoggerLevel );
    void SetReaderLogger( LoggerServer* pLoggerServer, LoggerClient::LoggerLevel loggerLevel );
    bool Initialize();
    void Terminate();
    int LockInterface( unsigned long serialNumber, unsigned short& rProductId, ReaderBase*& rpReaderBase );
    void UnlockInterface( int interfaceId );
    unsigned int GetAttachedReaders( unsigned long* pSerialNumberStorage, unsigned int serialNumberStorageEntries );

protected:
    virtual void ReaderAdded( const char* pNode, unsigned short productId, unsigned long serialNumber );
    virtual void DeviceRemoved( const char* pNode );

private:
    enum
    {
        INVALID_SERIAL_NUMBER = 0,
        READER_COUNT_MAX = 8
    };

    struct InterfaceDescriptor
    {
        unsigned int ProductId;
        unsigned long SerialNumber;
        bool IsLocked;
        ReaderBase* pReader;
    };
    // Copy constructor is invalid and not defined, declared here to
    // prevent the compiler generating the default definition
    ReaderFabric( const ReaderFabric& rOther );

    // Copy assignment operator is invalid and not defined, declared
    // here to prevent compiler generating the default definition
    ReaderFabric& operator=( const ReaderFabric& rOther );

    void RemoveDevice( InterfaceDescriptor* pNode );

    struct
    {
        LoggerServer* pServer;
        LoggerClient::LoggerLevel Level;
    } m_ReaderLogger;
    LoggerClient m_LoggerClient;
    InterfaceDescriptor m_InterfaceDescriptor[ READER_COUNT_MAX ];
    ::pthread_mutex_t m_InterfaceDescriptorLock;
};


#endif // READERFABRIC__HPP
