// Standard includes

// Vendor includes

// Project includes
#include "ReaderModelPrEh08.hpp"
#include "LoggerServerImpl.hpp"

// Using declarations


ReaderModelPrEh08* ReaderModelPrEh08::Create( const char* pNode, unsigned long serialNumber,
                                              LoggerServer* pLoggerServer = 0, LoggerClient::LoggerLevel loggerLevel = LoggerClient::LOGGER_LEVEL__ERROR )
{
    ReaderModelPrEh08* pReaderModelPrEh08 = new ReaderModelPrEh08;

    // Do not want to handle this through the exception mechanism - disable exceptions for memory allocation
    if ( 0 != pReaderModelPrEh08 )
    {
        pReaderModelPrEh08->SetLogger( pLoggerServer, loggerLevel );

        if ( pReaderModelPrEh08->Open( pNode, serialNumber, ReaderModelPrEh08::CONFIGURATION__SPEED, ReaderModelPrEh08::CONFIGURATION__PARITY_CODE,
                                       ReaderModelPrEh08::CONFIGURATION__STOP_BITS ) )
        {
            pReaderModelPrEh08->SetReadTimeout( CONFIGURATION__READ_TIMEOUT_IN_MILLISECONDS );
        }
        else
        {
            delete pReaderModelPrEh08;
            pReaderModelPrEh08 = 0;
        }
    }

    return pReaderModelPrEh08;
}

void ReaderModelPrEh08::Delete( ReaderModelPrEh08* pReaderModelPrEh08 )
{
    if ( 0 != pReaderModelPrEh08 )
    {
        delete pReaderModelPrEh08;
    }
}

void ReaderModelPrEh08::SetLogger( LoggerServer* pLoggerServer, LoggerClient::LoggerLevel loggerLevel )
{
    if ( 0 != pLoggerServer )
    {
        m_LoggerClient.AttachLoggerServer( pLoggerServer );
    }
    m_LoggerClient.SetLevel( loggerLevel );
}

bool ReaderModelPrEh08::PortReinit()
{
    return this->Reopen();
}

bool ReaderModelPrEh08::ProcessGetInfo( Response::GetInfo& rGetInfo )
{
    bool isSuccess = false;

    Command::GetInfo commandGetInfo;
    commandGetInfo.Header.Code = 0x3F;
    commandGetInfo.Header.Length = 0;

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandGetInfo ),
                               reinterpret_cast < unsigned char* > ( &rGetInfo ), sizeof ( rGetInfo ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

bool ReaderModelPrEh08::ProcessSetBeepAndBlink( bool beepOn, bool blinkOn, Response::SetBeepAndBlink& rSetBeepAndBlink )
{
    bool isSuccess = false;

    Command::SetBeepAndBlink commandSetBeepAndBlink;
    commandSetBeepAndBlink.Header.Code = 0x30;
    commandSetBeepAndBlink.Header.Length = sizeof ( Command::SetBeepAndBlink ) - sizeof ( Command::SetBeepAndBlink::Header );
    commandSetBeepAndBlink.Configuration = ( beepOn ? 0x08 : 0x00 ) | ( blinkOn ? 0x01 : 0x00 );

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandSetBeepAndBlink ),
                               reinterpret_cast < unsigned char* > ( &rSetBeepAndBlink ), sizeof ( rSetBeepAndBlink ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

bool ReaderModelPrEh08::ProcessBeep( Response::Beep& rResponseBeep )
{
    bool isSuccess = false;

    Command::Beep commandBeep;
    commandBeep.Header.Code = 0x31;
    commandBeep.Header.Length = 0;

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandBeep ),
                               reinterpret_cast < unsigned char* > ( &rResponseBeep ), sizeof ( rResponseBeep ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

bool ReaderModelPrEh08::ProcessReadCard( Response::ReadCard& rResponseReadCard )
{
    bool isSuccess = false;

    Command::ReadCard commandReadCard;
    commandReadCard.Header.Code = 0x32;
    commandReadCard.Header.Length = 0;

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandReadCard ),
                               reinterpret_cast < unsigned char* > ( &rResponseReadCard ), sizeof ( rResponseReadCard ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

ReaderModelPrEh08::ReaderModelPrEh08()
    : ReaderBase( m_LoggerClient, &( m_CommandPacket[ 0 ] ) ),
      m_LoggerClient( LoggerServerImpl::LOGGER_CLIENT_ID__READER_MODEL_PR_EH08 )
{
}

ReaderModelPrEh08::~ReaderModelPrEh08()
{
}
