// Standard includes

// Vendor includes

// Project includes
#include "ReaderModelPrp08.hpp"
#include "LoggerServerImpl.hpp"

// Using declarations


ReaderModelPrp08* ReaderModelPrp08::Create( const char* pNode, unsigned long serialNumber,
                                            LoggerServer* pLoggerServer = 0, LoggerClient::LoggerLevel loggerLevel = LoggerClient::LOGGER_LEVEL__ERROR )
{
    ReaderModelPrp08* pReaderModelPrp08 = new ReaderModelPrp08;

    // Do not want to handle this through the exception mechanism - disable exceptions for memory allocation
    if ( 0 != pReaderModelPrp08 )
    {
        pReaderModelPrp08->SetLogger( pLoggerServer, loggerLevel );

        if ( pReaderModelPrp08->Open( pNode, serialNumber, ReaderModelPrp08::CONFIGURATION__SPEED, ReaderModelPrp08::CONFIGURATION__PARITY_CODE,
                                      ReaderModelPrp08::CONFIGURATION__STOP_BITS ) )
        {
            pReaderModelPrp08->SetReadTimeout( CONFIGURATION__READ_TIMEOUT_IN_MILLISECONDS );
        }
        else
        {
            delete pReaderModelPrp08;
            pReaderModelPrp08 = 0;
        }
    }

    return pReaderModelPrp08;
}

void ReaderModelPrp08::Delete( ReaderModelPrp08* pReaderModelPrp08 )
{
    if ( 0 != pReaderModelPrp08 )
    {
        delete pReaderModelPrp08;
    }
}

void ReaderModelPrp08::SetLogger( LoggerServer* pLoggerServer, LoggerClient::LoggerLevel loggerLevel )
{
    if ( 0 != pLoggerServer )
    {
        m_LoggerClient.AttachLoggerServer( pLoggerServer );
    }
    m_LoggerClient.SetLevel( loggerLevel );
}

bool ReaderModelPrp08::PortReinit()
{
    return this->Reopen();
}

bool ReaderModelPrp08::ProcessGetInfo( Response::GetInfo& rGetInfo )
{
    bool isSuccess = false;

    Command::GetInfo commandGetInfo;
    commandGetInfo.Header.Code = 0x3F;
    commandGetInfo.Header.Length = 0;

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandGetInfo ),
                               reinterpret_cast < unsigned char* > ( &rGetInfo ), sizeof ( rGetInfo ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

bool ReaderModelPrp08::ProcessSetBeepAndBlink( bool beepOn, bool blinkOn, Response::SetBeepAndBlink& rSetBeepAndBlink )
{
    bool isSuccess = false;

    Command::SetBeepAndBlink commandSetBeepAndBlink;
    commandSetBeepAndBlink.Header.Code = 0x30;
    commandSetBeepAndBlink.Header.Length = sizeof ( Command::SetBeepAndBlink ) - sizeof ( Command::SetBeepAndBlink::Header );
    commandSetBeepAndBlink.Configuration = ( beepOn ? 0x02 : 0x00 ) | ( blinkOn ? 0xC8 : 0xC0 );

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandSetBeepAndBlink ),
                               reinterpret_cast < unsigned char* > ( &rSetBeepAndBlink ), sizeof ( rSetBeepAndBlink ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

bool ReaderModelPrp08::ProcessBeep( Response::Beep& rResponseBeep )
{
    bool isSuccess = false;

    Command::Beep commandBeep;
    commandBeep.Header.Code = 0x31;
    commandBeep.Header.Length = 0;

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandBeep ),
                               reinterpret_cast < unsigned char* > ( &rResponseBeep ), sizeof ( rResponseBeep ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

bool ReaderModelPrp08::ProcessRf( unsigned char cardType, unsigned char rfState, Response::Rf& rResponseRf )
{
    bool isSuccess = false;

    Command::Rf commandRf;
    commandRf.Header.Code = 0x3B;
    commandRf.Header.Length = sizeof ( Command::Rf ) - sizeof ( Command::Rf::Header );
    commandRf.CardType = cardType;
    commandRf.RfState = rfState;

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandRf ),
                               reinterpret_cast < unsigned char* > ( &rResponseRf ), sizeof ( rResponseRf ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

bool ReaderModelPrp08::ProcessReadCard( unsigned char requestType, Response::ReadCard& rResponseReadCard )
{
    bool isSuccess = false;

    Command::ReadCard commandReadCard;
    commandReadCard.Header.Code = 0x49;
    commandReadCard.Header.Length = sizeof ( Command::ReadCard ) - sizeof ( Command::Rf::Header );
    commandReadCard.RequestType = requestType;

    if ( this->ProcessCommand( reinterpret_cast < unsigned char* > ( &commandReadCard ),
                               reinterpret_cast < unsigned char* > ( &rResponseReadCard ), sizeof ( rResponseReadCard ) ) )
    {
        isSuccess = true;
    }

    return isSuccess;
}

ReaderModelPrp08::ReaderModelPrp08()
    : ReaderBase( m_LoggerClient, &( m_CommandPacket[ 0 ] ) ),
      m_LoggerClient( LoggerServerImpl::LOGGER_CLIENT_ID__READER_MODEL_PRP_08 )
{
}

ReaderModelPrp08::~ReaderModelPrp08()
{
}
