#ifndef READERMODELPRP08__HPP
#define READERMODELPRP08__HPP

// Standard includes

// Vendor includes

// Project includes
#include "ReaderBase.hpp"
#include "Logger.hpp"


class ReaderModelPrp08 : public ReaderBase
{
public:
    enum
    {
        PRODUCT_ID = 0xE3B0
    };

    // Command structures
    union Command
    {
        struct GetInfo
        {
            ReaderBase::CommandHeader Header;
        } __attribute__ ( ( __packed__ ) ) HelperGetInfo;

        struct SetBeepAndBlink
        {
            ReaderBase::CommandHeader Header;
            unsigned char Configuration;
        } __attribute__ ( ( __packed__ ) ) HelperSetBeepAndBlink;

        struct Beep
        {
            ReaderBase::CommandHeader Header;
        } __attribute__ ( ( __packed__ ) ) HelperBeep;

        struct Rf
        {
            enum
            {
                CARD_TYPE__NO_CHANGE = 0x00,
                CARD_TYPE__MIFARE = 0x05,

                RF_STATE__NO_CHANGE = 0x00,
                RF_STATE__OFF = 0x01,
                RF_STATE__LOW_POWER = 0x02,
                RF_STATE__HIGH_POWER = 0x03,
            };

            ReaderBase::CommandHeader Header;
            unsigned char CardType;
            unsigned char RfState;
        } __attribute__ ( ( __packed__ ) ) HelperRf;

        struct ReadCard
        {
            enum
            {
                REQUEST_TYPE__HALTED_CARD_ACTIVATE = 0x52,
                REQUEST_TYPE__IDLE_CARD_ACTIVATE = 0x26
            };

            ReaderBase::CommandHeader Header;
            unsigned char RequestType;
        } __attribute__ ( ( __packed__ ) ) HelperReadCard;
    };

    // Response structures
    union Response
    {
        struct GetInfo
        {
            enum ProcessorType
            {
                PROCESSOR_TYPE__ATMEGA_8535 = 0,
                PROCESSOR_TYPE__ATMEGA_08   = 1,
                PROCESSOR_TYPE__ATMEGA_16   = 2,
                PROCESSOR_TYPE__ATMEGA_32   = 3,
                PROCESSOR_TYPE__ATMEGA_64   = 4,
                PROCESSOR_TYPE__ATMEGA_128  = 5,
                PROCESSOR_TYPE__MICROCHIP   = 6
            };

            enum ReaderType
            {
                READER_MODE__ISO_14443A = 0,
                READER_MODE__ISO_14443B = 1,
                READER_MODE__MIFARE     = 4,
                READER_MODE__EM         = 5,
                READER_MODE__HID        = 6,
                READER_MODE__DALLAS     = 7,
                READER_MODE__EM_HID     = 8
            };

            enum RfState
            {
                RF_STATE__OFF  = 0,
                RF_STATE__LOW  = 1,
                RF_STATE__HIGH = 2
            };

            ReaderBase::ResponseHeader Header;
            unsigned char ProductInfo[ 5 ];
            unsigned char ChipSerialNumber[ 4 ];
            struct
            {
                unsigned char One;
                unsigned char Two;
            } __attribute__ ( ( __packed__ ) ) VersionAscii;
            union
            {
                struct
                {
                    unsigned char reserved  : 1;
                    unsigned char ethernet  : 1;
                    unsigned char usb       : 1;
                    unsigned char parsec_v2 : 1;
                    unsigned char parsec_v1 : 1;
                    unsigned char wiegand   : 1;
                    unsigned char rs_485    : 1;
                    unsigned char rs_232    : 1;
                } __attribute__ ( ( __packed__ ) ) Bits;
                unsigned char All;
            } __attribute__ ( ( __packed__ ) ) SupportedInterfaces;
            unsigned char ProcessorType;
            unsigned char ReaderMode;
            unsigned char RfState;
            unsigned char ReaderAddress;
        } __attribute__ ( ( __packed__ ) ) HelperGetInfo;

        struct SetBeepAndBlink
        {
            ReaderBase::ResponseHeader Header;
        } __attribute__ ( ( __packed__ ) ) HelperSetBeepAndBlink;

        struct Beep
        {
            ReaderBase::ResponseHeader Header;
        } __attribute__ ( ( __packed__ ) ) HelperBeep;

        struct Rf
        {
            ReaderBase::ResponseHeader Header;
        } __attribute__ ( ( __packed__ ) ) HelperRf;

        struct ReadCard
        {
            ReaderBase::ResponseHeader Header;
            struct
            {
                unsigned char Low;
                unsigned char High;
            } __attribute__ ( ( __packed__ ) ) Atqa;
            unsigned char Sak;
            unsigned char UidLength;
            unsigned char Uid[ 16 ];
        } __attribute__ ( ( __packed__ ) ) HelperReadCard;
    };

    static ReaderModelPrp08* Create( const char* pNode, unsigned long serialNumber, LoggerServer* pLoggerServer, LoggerClient::LoggerLevel loggerLevel );
    static void Delete( ReaderModelPrp08* pReaderModelPrp08 );

    void SetLogger( LoggerServer* pLoggerServer, LoggerClient::LoggerLevel loggerLevel );
    bool PortReinit();
    bool ProcessGetInfo( Response::GetInfo& rGetInfo );
    bool ProcessSetBeepAndBlink( bool beepOn, bool blinkOn, Response::SetBeepAndBlink& rSetBeepAndBlink );
    bool ProcessBeep( Response::Beep& rResponseBeep );
    bool ProcessRf( unsigned char cardType, unsigned char rfState, Response::Rf& rResponseRf );
    bool ProcessReadCard( unsigned char requestType, Response::ReadCard& rResponseReadCard );

protected:

private:
    enum Configuration
    {
        CONFIGURATION__SPEED = 115200,
        CONFIGURATION__PARITY_CODE = 'N',
        CONFIGURATION__STOP_BITS = 1,
        CONFIGURATION__READ_TIMEOUT_IN_MILLISECONDS = 1000
    };

    // Copy constructor is invalid and not defined, declared here to
    // prevent the compiler generating the default definition
    ReaderModelPrp08( const ReaderModelPrp08& rOther );

    // Copy assignment operator is invalid and not defined, declared
    // here to prevent compiler generating the default definition
    ReaderModelPrp08& operator=( const ReaderModelPrp08& rOther );

    ReaderModelPrp08();
    virtual ~ReaderModelPrp08();

    LoggerClient m_LoggerClient;
    unsigned char m_CommandPacket[ sizeof ( ReaderBase::PacketHeader ) + sizeof ( Command ) + sizeof ( ReaderBase::PacketTail ) ];
};


#endif // READERMODELPRP08__HPP
