#ifndef READERPLUGANDPLAY__HPP
#define READERPLUGANDPLAY__HPP

// Standard includes
#include <libudev.h>

// Vendor includes

// Project includes
#include "Logger.hpp"


class ReaderPlugAndPlay : public Thread
{
public:
    ReaderPlugAndPlay();
    virtual ~ReaderPlugAndPlay();

protected:
    void SetLogger( LoggerServer* pLoggerServer, LoggerClient::LoggerLevel loggerLevel );
    bool Open();
    void ScheduleClose();
    bool IsRunning() const;

    virtual void ReaderAdded( const char* pNode, unsigned short productId, unsigned long serialNumber ) = 0;
    virtual void DeviceRemoved( const char* pNode ) = 0;

private:
    enum
    {
        // Control pipe identifiers
        PIPE_CHANNEL__READ = 0,
        PIPE_CHANNEL__WRITE,
        PIPE_CHANNEL__MAX
    };

    // Copy constructor is invalid and not defined, declared here to
    // prevent the compiler generating the default definition
    ReaderPlugAndPlay( const ReaderPlugAndPlay& rOther );

    // Copy assignment operator is invalid and not defined, declared
    // here to prevent compiler generating the default definition
    ReaderPlugAndPlay& operator=( const ReaderPlugAndPlay& rOther );

    virtual void Run( void* pUserData );

    void ReleaseResorces();
    void ScanBus();
    void ProcessTtyDevice( ::udev_device* pDevice );
    bool DecodeSerialUtf8( const char* pSerialUtf8, unsigned short& rProductId, unsigned long& rSerialNumber );

    LoggerClient m_LoggerClient;
    ::udev* m_pUdevContext;
    ::udev_monitor* m_pUdevMonitor;
    volatile bool m_IsRunning;
    int m_Pipe[ PIPE_CHANNEL__MAX ];
};


#endif // READERPLUGANDPLAY__HPP
