// Standard includes

// Vendor includes

// Project includes
#include "Thread.hpp"

// Using declarations


Thread::Thread()
    : m_State( THREAD__INITIAL ),
      m_ThreadId( 0 ),
      m_pUserData( 0 )
{
}

Thread::~Thread()
{
    this->ThreadStop();
}

bool Thread::ThreadStart( void* pUserData )
{
    if ( THREAD__INITIAL == m_State )
    {
        m_pUserData = pUserData;

        // Semaphore to block on when thread is created
        if ( 0 == ::sem_init( &m_Semaphore, 0, 0 ) )
        {
            // Thread is created as joinable by default - do not set attributes
            m_State = THREAD__RUNNING;
            if ( 0 != ::pthread_create( &m_ThreadId, 0, &RunStarter, this ) )
            {
                ::sem_destroy( &m_Semaphore );

                m_State = THREAD__INITIAL;
            }
        }
    }

    return ( THREAD__RUNNING == m_State );
}

void Thread::ThreadStop()
{
    if ( THREAD__RUNNING == m_State )
    {
        m_State = THREAD__STOPPED;

        ::sem_post( &m_Semaphore );

        ::pthread_join( m_ThreadId, 0 );

        ::sem_destroy( &m_Semaphore );

        m_State = THREAD__INITIAL;
    }
}

bool Thread::ThreadActivate()
{
    if ( THREAD__RUNNING == m_State )
    {
        ::sem_post( &m_Semaphore );
    }

    return ( THREAD__RUNNING == m_State );
}

void* Thread::RunStarter( void* pThread )
{
    while ( true )
    {
        ::sem_wait( &( reinterpret_cast < Thread* > ( pThread )->m_Semaphore ) );

        if ( THREAD__STOPPED == reinterpret_cast < Thread* > ( pThread )->m_State )
        {
            break;
        }

        reinterpret_cast < Thread* > ( pThread )->Run( reinterpret_cast < Thread* > ( pThread )->m_pUserData );
    }

    return 0;
}
