#ifndef THREAD__HPP
#define THREAD__HPP

// Standard includes
#include <pthread.h>
#include <semaphore.h>

// Vendor includes

// Project includes


class Thread
{
public:
    Thread();
    virtual ~Thread();

    bool ThreadStart( void* pUserData );
    void ThreadStop();

protected:
    virtual void Run( void* pUserData ) = 0;
    bool ThreadActivate();

private:
    enum ThreadStatus
    {
        THREAD__INITIAL = 0,
        THREAD__RUNNING,
        THREAD__STOPPED
    };

    // Copy constructor is invalid and not defined, declared here to
    // prevent the compiler generating the default definition
    Thread( const Thread& rOther );

    // Copy assignment operator is invalid and not defined, declared
    // here to prevent compiler generating the default definition
    Thread& operator=( const Thread& rOther );

    static void* RunStarter( void* pThread );

    volatile ThreadStatus m_State;
    ::sem_t m_Semaphore;
    ::pthread_t m_ThreadId;
    void* m_pUserData;
};


#endif // THREAD__HPP
