#ifndef USBSERIAL__HPP
#define USBSERIAL__HPP

// Standard includes
#include <sys/poll.h>
#include <termios.h>

// Vendor includes

// Project includes


class UsbSerial
{
public:
    enum ErrorCode
    {
        USB_SERIAL_ERROR__SUCCESS = 0,
        USB_SERIAL_ERROR__DEVICE_ALREADY_OPENED = -1,
        USB_SERIAL_ERROR__PARAMETER_ERROR = -2,
        USB_SERIAL_ERROR__DEVICE_NOT_FOUND = -3,
        USB_SERIAL_ERROR__DEVICE_OPEN_ERROR = -4,
        USB_SERIAL_ERROR__DEVICE_NOT_OPENED = -5,
        USB_SERIAL_ERROR__DEVICE_IO_OPERATION_ERROR = -6,
        USB_SERIAL_ERROR__DEVICE_IO_OPERATION_TIMEOUT = -7
    };

    UsbSerial();
    virtual ~UsbSerial();

    bool Open( const char* pNode, unsigned int speed, char parityCode, unsigned char stopBits );
    void Close();
    bool Read( unsigned char* pBuffer, unsigned int expectedLength, int timeToWaitInMilliseconds );
    bool Write( unsigned char* pBuffer, unsigned int length );
    ErrorCode GetLastError() const;
    const char* GetLastErrorString() const;

protected:

private:
    struct SpeedMapRecord
    {
        unsigned int Speed;
        unsigned int SpeedCode;
    };

    // Copy constructor is invalid and not defined, declared here to
    // prevent the compiler generating the default definition
    UsbSerial( const UsbSerial& rOther );

    // Copy assignment operator is invalid and not defined, declared
    // here to prevent compiler generating the default definition
    UsbSerial& operator=( const UsbSerial& rOther );

    static const SpeedMapRecord s_SpeedMap[];
    ::pollfd m_PollFd;
    ::termios m_TtyControl;
    ErrorCode m_LastError;
};


#endif // USBSERIAL__HPP
