#ifndef READERINTERFACE_H
#define READERINTERFACE_H

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/*-------------------------- HEADER FILE INCLUDES ----------------------------*/
/*                   Use of this section is not recommended                   */
/*------------------------------ CONSTANTS -----------------------------------*/
#ifndef FALSE
#define FALSE ( 0 )
#endif /* TRUE */

#ifndef TRUE
#define TRUE ( 1 )
#endif /* TRUE */

/*----------------------------- ENUMERATIONS ---------------------------------*/
typedef enum
{
    PROCESSOR_TYPE__ATMEGA_8535 = 0,
    PROCESSOR_TYPE__ATMEGA_08   = 1,
    PROCESSOR_TYPE__ATMEGA_16   = 2,
    PROCESSOR_TYPE__ATMEGA_32   = 3,
    PROCESSOR_TYPE__ATMEGA_64   = 4,
    PROCESSOR_TYPE__ATMEGA_128  = 5,
    PROCESSOR_TYPE__MICROCHIP   = 6

} Processor_type;

typedef enum
{
    READER_MODE__ISO_14443A = 0,
    READER_MODE__ISO_14443B = 1,
    READER_MODE__MIFARE     = 4,
    READER_MODE__EM         = 5,
    READER_MODE__HID        = 6,
    READER_MODE__DALLAS     = 7,
    READER_MODE__EM_HID     = 8

} Reader_mode;

typedef enum
{
    RF_STATE__OFF  = 0,
    RF_STATE__LOW  = 1,
    RF_STATE__HIGH = 2

} Rf_state;

typedef enum
{
    CARD_TYPE__MIFARE = 0,
    CARD_TYPE__EM_MARINE,
    CARD_TYPE__HID

} Card_type;

/*--------------------------- BASIC DATA TYPES -------------------------------*/
typedef void (*debug_log_func_t)(
    const char* subsystem_name_ptr,
    const char* debug_level_name_ptr,
    char* log_text_ptr );

/*---------------------- STRUCTURE/UNION DATA TYPES --------------------------*/
typedef struct
{
    unsigned char product_info[ 5 ];
    unsigned char chip_serial_number[ 4 ];
    struct
    {
        unsigned char one;
        unsigned char two;

    } version_ascii;
    union
    {
        struct
        {
            unsigned char reserved  : 1;
            unsigned char ethernet  : 1;
            unsigned char usb       : 1;
            unsigned char parsec_v2 : 1;
            unsigned char parsec_v1 : 1;
            unsigned char wiegand   : 1;
            unsigned char rs_485    : 1;
            unsigned char rs_232    : 1;
        } bits;
        unsigned char all;

    } supported_interfaces;
    Processor_type processor_type;
    Reader_mode reader_mode;
    Rf_state rf_state;
    unsigned char reader_address;

} Get_info_response_t;

typedef struct
{
    Card_type card_type;
    union
    {
        struct
        {
            struct
            {
                unsigned char low;
                unsigned char high;

            } atqa;
            unsigned char sak;
            unsigned int uid_length;
            unsigned char uid[ 16 ];

        } mifare;

        struct
        {
            unsigned int length;
            unsigned char data[ 8 ];

        } em_marine_hid;

    } card;

} Read_card_response_t;

/*------------------------ FUNCTION PROTOTYPE(S) -----------------------------*/
void reader_set_log_func(
    debug_log_func_t debug_log_func_ptr );

void reader_debug_level(
    int fabric_debug_level,
    int plug_and_play_debug_level,
    int reader_object_debug_level );

/* Return 0 in case of success, -1 otherwise */
int reader_interface_init();

void reader_interface_term();

/* Return the number of connected readers */
unsigned int reader_get_attached(
    unsigned long* serial_number_storage_ptr,
    unsigned int serial_number_storage_entries );

/* Return 0 in case of success, -1 otherwise */
int reader_get_info(
    unsigned long serial_number,
    Get_info_response_t* get_info_response_ptr );

/* Return 0 in case of success, -1 otherwise */
int reader_set_beep_and_blink_mode(
    unsigned long serial_number,
    unsigned char beep_on,
    unsigned char blink_on );

/* Return 0 in case of success, -1 otherwise */
int reader_beep(
    unsigned long serial_number);

void reader_set_wiegand_26( unsigned char wiegand_26_on );

/* Return 0 in case of success, -1 otherwise */
int reader_read_card(
    unsigned long serial_number,
    unsigned long* card_number_ptr,
    Read_card_response_t* read_card_response_ptr );

/*----------------------------- GLOBAL DATA ----------------------------------*/
/*                   Use of this section is not recommended                   */
/*------------------------------- MACROS -------------------------------------*/

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif /* READERINTERFACE_H */
